@@ -334,7 +334,7 @@ def import_ont_calls_for_read_level(toolname, call_encode, callfn, score_cutoff,
     ## ont_call0 is raw ont-calls, too large, it will be cut to only with bs-seq, named ont_call1
     ont_call0 = import_call(callfn, call_encode, baseFormat=baseFormat, include_score=True, siteLevel=False,
                             filterChr=args.chrSet, using_cache=using_cache, enable_cache=enable_cache,
-                            cache_dir=ds_cache_dir, toolname=toolname, score_cutoff=score_cutoff)
+                            cache_dir=ds_cache_dir, toolname=toolname, raw_cutoff=score_cutoff)
     sites_summary = {'Dataset': dsname,
                      'Method': toolname,
                      'Sites': len(ont_call0),
@@ -531,7 +531,7 @@ def parse_arguments():
                         help="running prefix/output folder name, such as MethPerf-Dataset_WGBS_2Reps",
                         required=True)
     parser.add_argument('--calls', nargs='+',
-                        help='all ONT call results <tool-name>:<file-encode>:<file-name> seperated by space, tool-name/file-encode can be Nanopolish, Megalodon, DeepSignal, Guppy, Tombo, METEORE, DeepMod, NANOME',
+                        help='all ONT call results <tool-name>:<file-encode>:<file-name>[<cutoff0>:<cutoff1>] seperated by space, tool-name/file-encode can be Nanopolish, Megalodon, DeepSignal, Guppy, Tombo, METEORE, DeepMod, NANOME. The optional cutoff0 and cutoff1 can be specified if user wants to change default cutoff values, i.e., Nanpolish uses -2:2, Megalodon uses 0.2:0.8, etc..',
                         required=True)
     parser.add_argument('--bgtruth', type=str,
                         help="background truth file <encode-type>:<file-name1>;<file-name2>, encode-type can be 'encode' or 'bismark'",
@@ -759,7 +759,7 @@ if __name__ == '__main__':
             elif len(callstr.split(':')) == 5:
                 toolname, callencode, callfn, cutoff1, cutoff2 = callstr.split(':')
                 cutoff1 = float(cutoff1)
-                cutoff2 = float(cutoff1)
+                cutoff2 = float(cutoff2)
                 score_cutoff = (cutoff1, cutoff2)
         except:
             raise Exception(f"--calls params is not correct: {callstr}")
